seqs1 = ['AAAKILQTPK',
     'AAFDGGFTVK',
     'AALSTALSEK',
     'AASSHVPIQK',
     'AGNFHITVGK',
     'ALADAILINK',
     'ALADFLPVMK',
     'ALADIILINK',
     'AIADSPTGQK',
     'ALAEKFKELK',
     'ALALGVTLLK',
     'ALAPILLATL',
     'ALAPSTMKIK',
     'AIAQPATTAK',
     'ALASLVSAPK',
     'ALDENQRLAK',
     'ALDTKIGRYK',
     'ALFAKPLKGK',
     'ALFASGLIHR',
     'AIFFLPDSSK',
     'AIFHGWVLQK',
     'ALFKPGAWFK',
     'AIFSVYVVNK',
     'ALKNPPINTK',
     'ALKSINNAEK',
     'ALLAYTLGVK',
     'AIIDHIFASK',
     'ALIDYENSNK',
     'ALIEELLLYK',
     'AILEGDFSTK',
     'ALIGFPSVGK',
     'AILGYRSVGK',
     'ALLPAGFRQK',
     'ALIPGTLIKK',
     'ALLPWPFKQK',
     'ALLPWPFRNK',
     'ALIQQATTVK',
     'ALLSMTFPLK',
     'ALNDVTAARK',
     'AINEVVTREY',
     'AINPINTFTK',
     'ALNSAEKITK',
     'ALPSRILLWK',
     'AIQNPTMINK',
     'ALQSITAGQK',
     'ALRDNSTMGY',
     'ALRETGVIEK',
     'ALSAVSTQQK',
     'AISSFIAYQK',
     'ALSSIISRQK']

seqs2 = ['VVNDERFVSK',
 'VVNIVPVIAK',
 'VVPEMTEILK',
 'VVPKAAIVAR',
 'VVPKKLTIGK',
 'VVPLEEGLNK',
 'VVQEPGQVFK',
 'VVQTLVAKEK',
 'VVSESLTPER',
 'VVSSGPSVLR',
 'VVSSVVKVKK',
 'VVVDPIQSVK',
 'VVVFPAGLLQ',
 'VVVGAGGVGK',
 'VVVGDGAVGK',
 'VVVGNGAVGK',
 'VVVNNAGILR',
 'VVVPPLPGKA',
 'VVYEDIFDAK',
 'VVYGGKSTIR',
 'VVYPPPVDPK',
 'VVYTYTGKRI',
 'VVYVGGILTK',
 'VYGQKEIHRK',
 'VYPFPGPIPN',
 'WSSSRPIVQK',
 'YAFMGTPVQK',
 'YALPHAILRL',
 'YGSAPRTPSK',
 'YGYSNPKILK',
 'YKYIQELWRK',
 'YIIDFGLAKK',
 'YIQKNVQLYK',
 'YIQTDHLFFK',
 'YIYDGELVSK',
 'YSAFGVIIEK',
 'YSKEGWEYVK',
 'YSNYGPPQGK',
 'YSYGDFLLGK',
 'YSYQYTVANK',
 'YSYYNLILKK',
 'YTASLVGMIK',
 'YTCSFCGKTK',
 'YTFFTPGNLK',
 'YTFYSISQER',
 'YTLDPDTLRK',
 'YTLDVTTGQR',
 'YTIPSDFISK',
 'YTYEHDPITK',
 'YVLDDLIVAK']